@@ -1,7 +1,10 @@
 package Spreadsheet::HTML;
 use strict;
 use warnings FATAL => 'all';
-our $VERSION = '0.14';
+our $VERSION = '0.15';
+
+use Exporter 'import';
+our @EXPORT = qw( portrait generate landscape transpose flip mirror reverse earthquake tsunami );
 
 use Clone;
 use HTML::AutoTag;
@@ -9,34 +12,52 @@ use Math::Matrix;
 use Spreadsheet::HTML::File::Loader;
 
 sub portrait    { generate( @_ ) }
-sub generate    { _make_table( process( @_ ) ) }
+sub generate    { _make_table( _process( @_ ) ) }
 
 sub landscape   { transpose( @_ ) }
 sub transpose   {
-    my %args = process( @_ );
+    my %args = _process( @_ );
     $args{data} = [@{ Math::Matrix::transpose( $args{data} ) }];
     return _make_table( %args, tgroups => 0 );
 }
 
 sub flip   {
-    my %args = process( @_ );
+    my %args = _process( @_ );
     $args{data} = [ CORE::reverse @{ $args{data} } ];
     return _make_table( %args, tgroups => 0 );
 }
 
 sub mirror   {
-    my %args = process( @_ );
+    my %args = _process( @_ );
     $args{data} = [ map [ CORE::reverse @$_ ], @{ $args{data} } ];
     return _make_table( %args );
 }
 
 sub reverse   {
-    my %args = process( @_ );
+    my %args = _process( @_ );
     $args{data} = [ map [ CORE::reverse @$_ ], CORE::reverse @{ $args{data} } ];
     return _make_table( %args, tgroups => 0 );
 }
 
-sub process {
+sub earthquake   {
+    my %args = _process( @_ );
+    $args{data} = [ map [ CORE::reverse @$_ ], @{ Math::Matrix::transpose( $args{data} ) }];
+    return _make_table( %args, tgroups => 0 );
+}
+
+sub tsunami   {
+    my %args = _process( @_ );
+    $args{data} = [ map [ CORE::reverse @$_ ], CORE::reverse @{ Math::Matrix::transpose( $args{data} ) }];
+    return _make_table( %args, tgroups => 0 );
+}
+
+sub new {
+    my $class = shift;
+    my %attrs = ref($_[0]) eq 'HASH' ? %{+shift} : @_;
+    return bless { %attrs }, $class;
+}
+
+sub _process {
     my ($self,$data,$args) = _args( @_ );
 
     if ($self and $self->{is_cached}) {
@@ -47,13 +68,23 @@ sub process {
     my $max_cols = scalar @{ $data->[0] };
 
     if ($args->{layout}) {
-        $args->{encodes} = undef unless exists $args->{encodes}; 
+        $args->{encodes} = '' unless exists $args->{encodes}; 
         $args->{matrix} = 1 unless exists $args->{matrix};
         unless (exists $args->{table}) {
-            $args->{table}{role}          = 'presentation';
-            $args->{table}{border}        = 0;
-            $args->{table}{cellspacing}   = 0;
-            $args->{table}{cellpadding}   = 0;
+            $args->{table}{role} = 'presentation';
+            $args->{table}{$_}   = 0 for qw( border cellspacing cellpadding );
+        }
+    }
+
+    # headings is an alias for row_0
+    $args->{-row_0} = delete $args->{headings} if exists $args->{headings};
+
+    # headings to index mapping for column
+    my %index = ();
+    if ($#{ $data->[0] }) {
+        %index = map { '-' . $data->[0][$_] || '' => $_ } 0 .. $#{ $data->[0] };
+        for (grep /^-/, keys %$args) {
+            $args->{"-col_$index{$_}" } = delete $args->{$_} if exists $index{$_};
         }
     }
 
@@ -68,16 +99,34 @@ sub process {
             my $tag = (!$row and !($args->{headless} or $args->{matrix})) ? 'th' : 'td';
             my $val = $data->[$row][$col];
 
+            # -row_X
+            my $attr = $args->{$tag};
+            if (exists $args->{"-row_$row"}) {
+                if (ref($args->{"-row_$row"}) eq 'CODE') {
+                    $val = $args->{"-row_$row"}->($val);
+                } elsif (ref($args->{"-row_$row"}) eq 'HASH') {
+                    $attr = $args->{"-row_$row"};
+                }
+            }
+
+            # -col_X
+            unless ($row == 0) {
+                if (exists $args->{"-col_$col"}) {
+                    if (ref($args->{"-col_$col"}) eq 'CODE') {
+                        $val = $args->{"-col_$col"}->($val);
+                    } elsif (ref($args->{"-col_$col"}) eq 'HASH') {
+                        $attr = $args->{"-col_$col"};
+                    }
+                }
+            }
+
             # --empty
             do{ no warnings; $val =~ s/^\s*$/$empty/g };
 
-            # --headings
-            $val = $args->{headings}->($val) if !$row and ref($args->{headings}) eq 'CODE';
-
             $data->[$row][$col] = { 
                 tag => $tag, 
                 (defined( $val ) ? (cdata => $val) : ()), 
-                (defined( $args->{$tag} ) ? (attr => $args->{$tag}) : ()),
+                (defined( $attr ) ? (attr => $attr) : ()),
             };
         }
     }
@@ -92,12 +141,6 @@ sub process {
     return wantarray ? ( data => $data, %$args ) : $data;
 }
 
-sub new {
-    my $class = shift;
-    my %attrs = ref($_[0]) eq 'HASH' ? %{+shift} : @_;
-    return bless { %attrs }, $class;
-}
-
 sub _make_table {
     my %args = @_;
     $args{$_} ||= {} for qw( table tr thead tbody tfoot );
@@ -119,12 +162,40 @@ sub _make_table {
     my $encodes = exists $args{encodes} ? $args{encodes} : '';
     my $auto = HTML::AutoTag->new( encodes => $encodes, indent => $args{indent} );
 
-    # TODO: add caption back
-    # ( $args{caption} ? [caption => {}, $args{caption}] : () ),
+    my $caption;
+    if (ref($args{caption}) eq 'HASH') {
+        (my $cdata) = keys %{ $args{caption} };
+        (my $attr)  = values %{ $args{caption} };
+        $caption = { tag => 'caption', attr => $attr, cdata => $cdata };
+    } elsif (defined $args{caption} ) {
+        $caption = { tag => 'caption', cdata => $args{caption} };
+    } 
+
+    my @colgroup;
+    $args{col} = [ $args{col} ] if ref($args{col}) eq 'HASH';
+    if (ref($args{col}) eq 'ARRAY') {
+
+        @colgroup = ({
+            tag   => 'colgroup',
+            attr  => $args{colgroup},
+            cdata => [ map { tag => 'col', attr => $_ }, @{ $args{col} } ]
+        }); 
+
+    } else {
+
+        $args{colgroup} = [ $args{colgroup} ] if ref($args{colgroup}) eq 'HASH';
+        if (ref($args{colgroup}) eq 'ARRAY') {
+            @colgroup = map { tag => 'colgroup', attr => $_ }, @{ $args{colgroup} };
+        }
+    }
+
+
     return $auto->tag(
         tag => 'table',
         attr => $args{table},
         cdata => [
+            ( ref( $caption ) ? $caption : () ),
+            ( @colgroup ? @colgroup : () ),
             ( $args{tgroups} ? { tag => 'thead', attr => $args{thead}, cdata => $head_row }  : $head_row ),
             ( $args{tgroups} ? { tag => 'tfoot', attr => $args{tfoot}, cdata => $foot_row }  : $foot ? $foot_row : () ),
               $args{tgroups} ? { tag => 'tbody', attr => $args{tbody}, cdata => [@body_rows] } : @body_rows
@@ -132,13 +203,6 @@ sub _make_table {
     );
 }
 
-sub _element {
-    my ($tag, $content, $attr) = @_;
-    my $e = HTML::Element->new( $tag, %{ $attr || {} } );
-    $e->push_content( $content );
-    return $e;
-}
-
 sub _args {
     my ($self,$data,$args);
     $self = shift if UNIVERSAL::isa( $_[0], __PACKAGE__ );
@@ -185,16 +249,17 @@ Spreadsheet::HTML - Render HTML5 tables with ease.
     print $table->portrait;
     print $table->landscape;
 
-    # non OO
-    print Spreadsheet::HTML::portrait( $data );
-    print Spreadsheet::HTML::landscape( $data );
-
     # load from files (first table found)
     $table = Spreadsheet::HTML->new( file => 'data.xls', cache => 1 );
 
+    # non OO
+    use Spreadsheet::HTML qw( portrait landscape );
+    print portrait( $data );
+    print landscape( $data );
+
 =head1 DESCRIPTION
 
-THIS MODULE IS AN ALPHA RELEASE!
+THIS MODULE IS AN ALPHA RELEASE! Although we are very close to BETA.
 
 Renders HTML5 tables with ease. Provides a handful of distinctly
 named methods to control overall table orientation. These methods
@@ -203,18 +268,27 @@ what tags and attributes to use.
 
 =head1 METHODS
 
+All methods (except C<new>) are exportable as functions too. With the
+exception of C<new>, all methods return HTML as a scalar string. Any
+named parameters supplied to the method are applied to the data before
+any table rotations are performed. If it helps, work with the table
+using, for example, C<portrait()> until you have the parameters
+correct and then switch to C<landscape()> for the final product.
+All methods accept the same named parameters, although some methods
+override certain ones, hopefully in ways that make sense.
+
 =over 4
 
 =item * C<new( %args )>
 
   my $table = Spreadsheet::HTML->new( data => $data );
 
-Constructs object. Accepts named arguments (see ATTRIBUTES).
-Unless you give it an array of array refs. Or an array ref
-of array refs. Otherwise it expects named arguments. The
+Constructs object. Accepts named parameters (see PARAMETERS).
+Unless you give it a list of array refs. Or an array ref
+of array refs. Otherwise it expects named parameters. The
 most favorite being 'data' which is exactly an array ref
 of array refs. The first row will be treated as the headings
-unless you specify otherwise (see ATTRIBUTES).
+unless you specify otherwise (see PARAMETERS).
 
 =item * C<generate( %args )>
 
@@ -249,16 +323,23 @@ Columns are rendered right to left.
 Combines flip and mirror: flips the headings and
 data upside down and render columns right to left.
 
-=item * C<process( %args )>
+=item * C<earthquake( %args )>
 
-Returns processed data.
+Combines transpose/landscape with mirror.
+
+=item * C<tsunami( %args )>
+
+Combines transpose/landscape with reverse.
 
 =back
 
-=head1 ATTRIBUTES
+For most cases, C<portrait()> and C<landscape()> are all you need. They
+are simply aliases for C<generate()> and C<transpose()>, respectively.
 
-All methods/procedures accept named arguments.
-If named arguments are detected: the data has to be
+=head1 PARAMETERS
+
+All methods/procedures accept the same named parameters.
+If named parameters are detected: the data has to be
 an array ref assigned to the key 'data'. If no
 named args are detected then the parameter list is
 treated as the data itself, either an array containing
@@ -269,19 +350,25 @@ array references.
 
 =item * C<data: [ [], [], [], ... ]>
 
-The data to be rendered into table cells.
+The data to be rendered into table cells. Should be
+an array ref of array refs.
+
+  data => [["a".."c"],[1..3],[4..6],[7..9]]
 
 =item * C<file: $str>
 
 The name of the data file to read. Supported formats
 are XLS, CSV, JSON, YAML and HTML (first table found).
 
+  file => 'foo.json'
+
 =item * C<indent: $str>
 
-Render the table with whitespace indention. Defaults to
-undefined which produces no trailing whitespace to tags.
-Useful values are some number of spaces or tabs.  (see
-L<HTML::Element>::as_HTML).
+Render the table with nested indentation. Defaults to
+undefined which produces no indentation. Adds newlines
+when set to any value that is defined.
+
+  indent => '    '
 
 =item * C<encode: $str>
 
@@ -289,34 +376,81 @@ HTML Encode contents of td tags. Defaults to empty string
 which performs no encoding of entities. Pass a string like
 '<>&=' to perform encoding on any characters found. If the
 value is 'undef' then all unsafe characters will be
-encoded as HTML entites (see L<HTML::Element>::as_HTML).
+encoded as HTML entites.
+
+  encodes => '<>"'
 
 =item * C<empty: $str>
 
 Replace empty cells with this value. Defaults to &nbsp;
 Set value to undef to avoid any substitutions.
 
+  empty => '&#160;'
+
 =item * C<cache: 0 or 1>
 
+  cache => 1
+
 Preserve data after it has been processed (and loaded).
 
 =item * C<matrix: 0 or 1>
 
 Render the table with only td tags, no th tags, if true.
 
+  matrix => 1
+
 =item * C<layout: 0 or 1>
 
-Add W3C recommended table attributes, emit only <td> tags,
-no row padding or pruning, and force no HTML entity encoding
-in table cells.
+Layout tables are not recommended, but if you choose to
+use them you should label them as such. This adds W3C
+recommended layout attributes to the table tag and features:
+emiting only <td> tags, no padding or pruning of rows, forces
+no HTML entity encoding in table cells.
+
+  layout => 1
 
 =item * C<headless: 0 or 1>
 
-Render the table with without headings, if true.
+Render the table with without the headings row, if true. 
 
-=item * C<headings: sub { return function( shift ) }>
+  headless => 1
 
-Apply this anonymous subroutine to headers.
+=item * C<headings: \& or \%>
+
+Apply anonymous subroutine to each cell in headings row.
+
+  headings => sub {join(" ",map{ucfirst lc$_}split"_",shift)}
+
+Or apply hash ref as attributes:
+
+  headings => { class => 'some-class' }
+
+=item * C<-row_X: \& or \%>
+
+Apply this anonymous subroutine to row X. (0 index based)
+
+  -row_3 => sub { uc shift }
+
+Or apply hash ref as attributes:
+
+  -row_3 => { class => 'some-class' }
+
+=item * C<-col_X: \& or \%>
+
+Apply this anonymous subroutine to column X. (0 index based)
+
+  -col_4 => sub { sprintf "%02d", shift || 0 }
+
+Or apply hash ref as attributes:
+
+  -col_4 => { class => 'some-class' }
+
+You can alias any column number by the value of the heading
+name in that column:
+
+  -my_heading3 => sub { "<b>$_[0]"</b>" }
+
+  -my_heading3 => { class => 'special-row' }
 
 =item * C<tgroups: 0 or 1>
 
@@ -325,24 +459,56 @@ sections. The <tfoot> section is always found before
 the <tbody> section. Only available for C<generate()>,
 C<portrait()> and C<mirror()>.
 
-=item * C<table: \%args>
+  tgroups => 1
 
-=item * C<caption: $str>
+=item * C<caption: $str or \%>
 
-=item * C<thead: \%args>
+Caption is special in that you can either pass a string to
+be used as CDATA or a hash whose only key is the string
+to be used as CDATA:
 
-=item * C<tfoot: \%args>
+  caption => "Just Another Title"
 
-=item * C<tbody: \%args>
+  caption => { "With Attributes" => { align => "bottom" } }
 
-=item * C<tr: \%args>
+=item * C<colgroup: \@ or \%>
 
-=item * C<th: \%args>
+Add colgroup tag(s) to the table. Use an AoH for multiple.
 
-=item * C<td: \%args>
+=item * C<col: \@ or \%>
 
-Supply attributes to the HTML tags that compose the table.
-There is currently no support for col and colgroup.
+Add col tag(s) to the table. Use an AoH for multiple. Wraps
+tags within a colgroup tag.
+
+=item * C<table: \%>
+
+Apply these attributes to the table tag.
+
+  table => { class => 'spreadsheet' }
+
+=item * C<thead: \%>
+
+  thead => { style => 'background: color' }
+
+=item * C<tfoot: \%>
+
+  tfoot => { style => 'background: color' }
+
+=item * C<tbody: \%>
+
+  tbody => { style => 'background: color' }
+
+=item * C<tr: \%>
+
+  tr => { style => { background => [qw( color1 color2 )]' } }
+
+=item * C<th: \%>
+
+  th => { style => 'background: color' }
+
+=item * C<td: \%>
+
+  td => { style => 'background: color' }
 
 =back
 
@@ -460,22 +626,6 @@ trade off, so i will be working to develop my own solution, unless another CPAN
 module will suffice. Don't get me wrong, HTML::Tree is awesome and powerful.
 But i needs speed. And i could be wrong ... ;)
 
-This implementation is currently missing the following features:
-
-=over 4
-
-=item * build index map for columns
-
-=item * build auto col_XX and row_XX for item 1 (maybe ...)
-
-=item * emit col and colgroup tags
-
-=item * map client attrs and functions to cells by columns/rows
-
-=back
-
-You are encouraged to use L<DBIx::XHTML_Table> during the development of this module.
-
 =head1 GITHUB
 
 The Github project is L<https://github.com/jeffa/Spreadsheet-HTML>
